from django.db import models
from django.utils import timezone


class Comment(models.Model):
    user = models.ForeignKey("auth.User", on_delete=models.CASCADE)
    timestamp = models.DateTimeField(default=timezone.now)



from datetime import date, timedelta

from django import forms
from django.contrib.contenttypes.models import ContentType

from trackstats.models import Metric, StatisticByDate, StatisticByDateAndObject


class GraphByDateForm(forms.Form):
    statistic_model = StatisticByDate

    metric = forms.ModelChoiceField(queryset=None)
    from_date = forms.DateField(initial=date.today() - timedelta(days=7))
    to_date = forms.DateField(initial=date.today())

    def __init__(self, *args, **kwargs):
        super(GraphByDateForm, self).__init__(*args, **kwargs)
        self.fields["metric"].queryset = Metric.objects.all()

    def get_statistics(self):
        assert self.is_valid()
        stats = self.statistic_model.objects.narrow(
            from_date=self.cleaned_data["from_date"],
            to_date=self.cleaned_data["to_date"],
            metric=self.cleaned_data["metric"],
        )
        return stats.order_by("date")


class GraphByDateAndObjectForm(GraphByDateForm):
    statistic_model = StatisticByDateAndObject

    object_type = forms.ModelChoiceField(queryset=None, required=False)
    object_id = forms.IntegerField(required=False)

    def __init__(self, *args, **kwargs):
        super(GraphByDateAndObjectForm, self).__init__(*args, **kwargs)
        self.fields["object_type"].queryset = ContentType.objects.all()

    def get_statistics(self):
        stats = super(GraphByDateAndObjectForm, self).get_statistics()
        stats = stats.filter(object_type=self.cleaned_data["object_type"])
        object_id = self.cleaned_data["object_id"]
        if object_id:
            stats = stats.filter(object_id=object_id)
        return stats



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-01 14:40
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Domain",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "ref",
                    models.CharField(
                        help_text="Unique reference ID for this domain",
                        max_length=100,
                        unique=True,
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True, help_text="Short descriptive name", max_length=100
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Metric",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "ref",
                    models.CharField(
                        help_text="Unique reference ID for this metric within the domain",
                        max_length=100,
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True, help_text="Short descriptive name", max_length=100
                    ),
                ),
                ("description", models.TextField(blank=True, help_text="Description")),
                (
                    "domain",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="trackstats.Domain",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="StatisticByDate",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.BigIntegerField(null=True)),
                (
                    "period",
                    models.IntegerField(
                        choices=[
                            (86400, "Day"),
                            (604800, "Week"),
                            (2419200, "28 days"),
                            (2592000, "Month"),
                            (0, "Lifetime"),
                        ]
                    ),
                ),
                ("date", models.DateField(db_index=True)),
                (
                    "metric",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="trackstats.Metric",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Statistics by date",
                "verbose_name": "Statistic by date",
            },
        ),
        migrations.CreateModel(
            name="StatisticByDateAndObject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.BigIntegerField(null=True)),
                (
                    "period",
                    models.IntegerField(
                        choices=[
                            (86400, "Day"),
                            (604800, "Week"),
                            (2419200, "28 days"),
                            (2592000, "Month"),
                            (0, "Lifetime"),
                        ]
                    ),
                ),
                ("object_id", models.PositiveIntegerField()),
                ("date", models.DateField(db_index=True)),
                (
                    "metric",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="trackstats.Metric",
                    ),
                ),
                (
                    "object_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Statistics by date and object",
                "verbose_name": "Statistic by date and object",
            },
        ),
        migrations.AlterUniqueTogether(
            name="statisticbydateandobject",
            unique_together=set(
                [("date", "metric", "object_type", "object_id", "period")]
            ),
        ),
        migrations.AlterUniqueTogether(
            name="statisticbydate",
            unique_together=set([("date", "metric", "period")]),
        ),
        migrations.AlterUniqueTogether(
            name="metric",
            unique_together=set([("domain", "ref")]),
        ),
    ]



DATABASE_ENGINE = "sqlite3"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

INSTALLED_APPS = (
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.auth",
    "django.contrib.admin",
    "django.contrib.messages",
    "trackstats",
    "trackstats.tests",
)

MIDDLEWARE = (
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": (
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ),
        },
    }
]


USE_TZ = True

SECRET_KEY = "psst"



from django.contrib import admin
from django.template.response import TemplateResponse
from django.urls import re_path

from trackstats.admin.forms import GraphByDateAndObjectForm, GraphByDateForm
from trackstats.models import (
    Domain,
    Metric,
    StatisticByDate,
    StatisticByDateAndObject,
)


@admin.register(Domain)
class DomainAdmin(admin.ModelAdmin):
    list_display = ("ref", "name")


@admin.register(Metric)
class MetricAdmin(admin.ModelAdmin):
    search_fields = ("ref", "name")
    list_display = ("ref", "name")
    list_filter = ("domain",)


class StatisticGraphMixin(object):
    graph_slug = None
    graph_form_class = None

    def get_urls(self):
        urls = super(StatisticGraphMixin, self).get_urls()
        custom_urls = [
            re_path("^graph/$", self.graph, name="trackstats_graph_" + self.graph_slug)
        ]
        return custom_urls + urls

    def graph(self, request):
        context = dict(self.admin_site.each_context(request))
        if "to_date" in request.GET:
            form = self.graph_form_class(request.GET)
            if form.is_valid():
                stats = []
                for stat in form.get_statistics():
                    stats.append(
                        dict(
                            js_date="new Date({}, {}, {})".format(
                                stat.date.year, stat.date.month - 1, stat.date.day
                            ),
                            value=stat.value,
                        )
                    )
                context["statistics"] = stats
        else:
            stat = StatisticByDate.objects.last()
            initial = {}
            if stat:
                initial["metric"] = stat.metric
            form = self.graph_form_class(initial=initial)
        context["form"] = form
        return TemplateResponse(
            request, "trackstats/admin/{}/graph.html".format(self.graph_slug), context
        )


@admin.register(StatisticByDate)
class StatisticByDateAdmin(StatisticGraphMixin, admin.ModelAdmin):
    change_list_template = "trackstats/admin/by_date/change_list.html"
    graph_slug = "by_date"
    graph_form_class = GraphByDateForm

    ordering = ("-date",)
    list_display = ("date", "metric", "value")
    date_hierarchy = "date"
    list_filter = ("date", "period", "metric__domain", "metric")


@admin.register(StatisticByDateAndObject)
class StatisticByDateAndObjectAdmin(StatisticGraphMixin, admin.ModelAdmin):
    change_list_template = "trackstats/admin/by_date_and_object/change_list.html"
    graph_slug = "by_date_and_object"
    graph_form_class = GraphByDateAndObjectForm
    ordering = ("-date",)
    list_display = ("date", "metric", "object_type", "object_id", "value")
    date_hierarchy = "date"
    list_filter = ("date", "period", "metric__domain", "metric")


#            stat = StatisticByDate.objects.last()
#            initial = {}
#            if stat:
#                initial['metric'] = stat.metric
#                initial['subject_type'] = stat.subject_type
#            form = GraphForm(initial=initial)
#        context['form'] = form
#        return TemplateResponse(
#            request,
#            "trackstats/admin/graphs.html",
#            context)



# Generated by Django 4.0.4 on 2022-04-24 12:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("trackstats", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="domain",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="metric",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="statisticbydate",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="statisticbydateandobject",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



import random
from collections import Counter
from datetime import date, timedelta

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.utils import timezone

from trackstats.models import (
    Domain,
    Metric,
    Period,
    StatisticByDate,
    StatisticByDateAndObject,
)
from trackstats.tests.models import Comment
from trackstats.trackers import (
    CountObjectsByDateAndObjectTracker,
    CountObjectsByDateTracker,
)


def to_date(dt):
    return timezone.make_naive(dt).date()


class TrackersTestCase(TestCase):
    def setUp(self):
        self.User = get_user_model()
        self.users_domain = Domain.objects.register(ref="users")
        self.user_count = Metric.objects.register(
            domain=self.users_domain, ref="user_count"
        )
        self.expected_signups = {}
        dt = timezone.now() - timedelta(days=7)
        # TODO: Add timezone testing
        # dt.replace(hour=2)
        signups_lifetime = 0
        while to_date(dt) != date.today():
            signups_on_day = random.randint(1, 5)
            signups_lifetime += signups_on_day
            self.expected_signups[to_date(dt)] = {
                "lifetime": signups_lifetime,
                "day": signups_on_day,
            }
            for i in range(signups_on_day):
                self.User.objects.create(
                    username="user{}_{}".format(to_date(dt), i), date_joined=dt
                )
            dt += timedelta(days=1)
        self.expected_signups[date.today()] = self.expected_signups[
            date.today() - timedelta(days=1)
        ]

    def test_count_lifetime(self):
        CountObjectsByDateTracker(
            period=Period.LIFETIME, metric=self.user_count, date_field="date_joined"
        ).track(self.User.objects.all())
        stats = StatisticByDate.objects.narrow(
            metrics=[self.user_count], period=Period.LIFETIME
        )
        for stat in stats:
            self.assertEqual(stat.value, self.expected_signups[stat.date]["lifetime"])
        self.assertEqual(stats.count(), len(self.expected_signups))

    def test_count_daily(self):
        CountObjectsByDateTracker(
            period=Period.DAY, metric=self.user_count, date_field="date_joined"
        ).track(self.User.objects.all())
        stats = StatisticByDate.objects.narrow(
            metrics=[self.user_count], period=Period.DAY
        )
        for stat in stats:
            self.assertEqual(stat.value, self.expected_signups[stat.date]["day"])
        self.assertEqual(
            stats.count(),
            # Today is not in there due to group by.
            len(self.expected_signups) - 1,
        )


class ObjectTrackersTestCase(TestCase):
    def setUp(self):
        self.User = get_user_model()
        domain = Domain.objects.register(ref="comments")
        self.comment_count = Metric.objects.register(domain=domain, ref="comment_count")
        users = self.users = [
            self.User.objects.create(username="user{}".format(i)) for i in range(5)
        ]
        dt = timezone.now() - timedelta(days=7)
        self.expected_daily = {}
        self.expected_lifetime = Counter()
        while to_date(dt) <= date.today():
            for user in users:
                comment_count = random.randint(1, 5)
                for i in range(comment_count):
                    Comment.objects.create(timestamp=dt, user=user)
                self.expected_lifetime[(to_date(dt), user.pk)] = (
                    self.expected_lifetime[(to_date(dt) - timedelta(days=1), user.pk)]
                    + comment_count
                )
                self.expected_daily[(to_date(dt), user.pk)] = comment_count
            dt += timedelta(days=1)

    def test_count_lifetime(self):
        CountObjectsByDateAndObjectTracker(
            period=Period.LIFETIME,
            metric=self.comment_count,
            object_model=self.User,
            object_field="user",
            date_field="timestamp",
        ).track(Comment.objects.all())
        stats = StatisticByDateAndObject.objects.narrow(
            metrics=[self.comment_count], period=Period.LIFETIME
        )
        for stat in stats:
            self.assertEqual(
                self.expected_lifetime[(stat.date, stat.object.pk)], stat.value
            )
        self.assertEqual(stats.count(), len(self.expected_lifetime))

    def test_count_daily(self):
        CountObjectsByDateAndObjectTracker(
            period=Period.DAY,
            metric=self.comment_count,
            object_model=self.User,
            object_field="user",
            date_field="timestamp",
        ).track(Comment.objects.all())
        stats = StatisticByDateAndObject.objects.narrow(
            metric=self.comment_count, period=Period.DAY
        )
        for stat in stats:
            self.assertEqual(
                self.expected_daily[(stat.date, stat.object.pk)], stat.value
            )
        self.assertEqual(stats.count(), len(self.expected_daily))



#!/usr/bin/env python
from __future__ import unicode_literals

import codecs
import os

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand

import trackstats


class DjangoTests(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        from django.core import management

        DSM = "DJANGO_SETTINGS_MODULE"
        if DSM not in os.environ:
            os.environ[DSM] = "trackstats.tests.settings"
        management.execute_from_command_line()


def read_files(*filenames):
    """
    Output the contents of one or more files to a single concatenated string.
    """
    output = []
    for filename in filenames:
        f = codecs.open(filename, encoding="utf-8")
        try:
            output.append(f.read())
        finally:
            f.close()
    return "\n\n".join(output)


# Dynamically calculate the version based on trackstats.VERSION.
version = trackstats.__version__

setup(
    name="django-trackstats",
    version=version,
    url="http://github.com/pennersr/django-trackstats",
    description="Statistics storage for Django",
    long_description=read_files("README.rst"),
    author="Raymond Penners",
    author_email="raymond.penners@intenct.nl",
    platforms=["any"],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["django>=3.2.13", "pytz"],
    cmdclass={"test": DjangoTests},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    zip_safe=False,
)



# Generated by Django 4.1.7 on 2023-07-12 16:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("trackstats", "0003_alter_model_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="domain",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="metric",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="statisticbydate",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="statisticbydateandobject",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
